"""
A module for controlling and receiving data from a ROACH-based RFI monitoring spectrometer.

Author: Monika Obrocka
Email: mobrocka at ska.ac.za
Revisions:
2017-03-13 First release
"""
import numpy
import csv
import sqlite3

